function [likel q history amatsmall endogsmall] = updatemod(params,paramlabels,funname,observedlabels,upperbound,lowerbound,zdata)
% This function returns the likelihood associated with the model in funname
% and observed data in zdata for a choice of the parameters listed in
% paramlabels set at values listed in params.
% Observedlabels list the variables names whose data observations are
% packed in zdata. Upperbound and Lowerbound are vectors consistent with
% the ordering in params and paramlabels that are used to check that the
% choice of parameters at which to evaluate the likelihood are within a
% desired upperbound and lowerbound

%% Define parameters and matrices for state and observable transition
% equations.
%
% The state evolution equation takes the form:
% b_k = F b_{k-1} + q w_k.
% The observable equation takes the form:
% y_k = H' b_{k} + r e_k.

% implements penalty function to ensure function evaluation is within
% desired bounds
% check inputs and modify them if outside boundaries (impose penalty later)
origparams = params;
params = min(max(params,lowerbound),upperbound);

% obtain decision rule
eval(['[dr_ ys_ lgy_ lgx_]=',funname,'(paramlabels,params);'])


% check for stochastically singular variables if one is found get rid of it
% only works if model is stationary
%covbtmp = varsigma(amatsmall,bmatsmall*bmatsmall');
%[amatsmall, bmatsmall,endogsmall] = checkvar(covbtmp,endogsmall,amatsmall,bmatsmall);

nobserved = size(observedlabels);
nvars = size(dr_.ghx,1);
nshocks = size(dr_.ghu,2);
statevar_pos = (dr_.nstatic +1):(nvars-dr_.nfwrd);
reordered = lgy_(dr_.order_var,:);
statevarlist = reordered(statevar_pos,:);

f = dr_.ghx(statevar_pos,:);

q = dr_.ghu(statevar_pos,:);


nobserved = size(observedlabels,1);
nstatevars = size(f,1);
h = zeros(nobserved,nstatevars);
statebvars = [];
nstateobserved = 0;
for i=1:nobserved
    % check if observed variable is state variable (if so, it gets a 1
    % in the relevant position
    isstate = strmatch(observedlabels(i,:),statevarlist,'exact');
    % if the current variabe is not a state variable then
    % isstate is empty
    if isempty(isstate)
        % not a state variable
        
        % next, figure out if
        % fundamental innovations  hit the variable
        % contemporaneously, if so move it into the state equation
        % otherwise, keep it in the observation equation
        
        nostate = strmatch(observedlabels(i,:),reordered,'exact');
        if isempty(nostate)
            error('One observed variable is not in the model')
        else
            if min(zeros(1,nshocks)==dr_.ghu(nostate,:))==0
                % in this case the innovations hit the variable
                % contemporaneously
                % deal with this variables at the end
                statebvars = [statebvars;nostate];
            else
                nstateobserved = nstateobserved+1;
                h(nstateobserved,:) = dr_.ghx(nostate,:);
            end
        end
    else
        % the variable is a state variable
        % put a 1 in the relevant position
        h(i,isstate) = 1;
    end

end


% now deal with variables that were not state variables
% but were hit by innovations contemporaneously
% augment f, q, and h matrices appropriately.

nstatebvars = size(statebvars,2); 
f = [f zeros(nstatevars,nstatebvars) 
     dr_.ghx(statebvars,:) zeros(nstatebvars,nstatebvars)];

q = [q;dr_.ghu(statebvars,:)];

h = [h, zeros(nstateobserved,nstatebvars)
     zeros(nstatebvars,nstateobserved) eye(nstatebvars) ]; 

% Varcov of innovations (taking into account b matrix)
Q= q*q';

% guess for the state vector before first Kalman updating
xi10 = zeros(size(f,1),1);


% defines size of training sample
ntrains = 12;
[likel,errcode]= kalmanFilterMatlab(full(f), full(h'), full(zdata'), full(xi10), full(Q), ntrains);
history = [];
likel = likel +1000000*(sum(abs(params-origparams)))^2;

